<?php
require_once ('Aprobacion.php');

/**
 * @author Angel
 * @version 1.0
 * @created 20-ago-2013 02:56:38 p.m.
 */
class AprobacionDAO extends Aprobacion
{

	/**
	 * Devuelve todos los registros deaprobacion o solo el especificado por el id
	 * @param $data Valor entero para regresar un registro especifico. Si se omite, regresa todos los registros
	 * @return array
	 */
	public function listarAprobaciones($data = NULL){
		return $this->executeSP('stp_Aprobaciones(:id)', array('id' => $data));
	}

	/**
	 * Funcion que valida la informacion
	 * @param $data Array con la informacion capturada
	 * @return array
	 */
	public function validateData($data)	{
		$data = (object) $data;
		$result = false;
		$errors = '';
		$ids = '';

		if (empty($data->id_tipo_comprobante) || $data->id_tipo_comprobante == 0) {
			$errors .= "- Debe seleccionar un tipo de comprobance.\n";
			$ids .= ',#id_tipo_comprobante';
		}
		
		$aprobacion = $this->buscaAprobacion(
				array(
					'serie' => $data->serie)
				);

		if (!empty($aprobacion['id_aprobacion'])) {
			if(($aprobacion['folio_inicial'] == $data->folio_inicial) && ($aprobacion['id_aprobacion'] != $data->id_aprobacion)){
				$errors .= "- Folio inicial ya esta registrado para esa serie.\n";
				$ids .= ',#serie, #folio_inicial';
			}else{
				if($data->folio_inicial <= $aprobacion['folio_final'] && $aprobacion['id_aprobacion'] != $data->id_aprobacion){
					$errors .= "- Folio inicial se encuentra en un rango ya registrado para esa serie.\n";
					$ids .= ',#serie, #folio_inicial';
				}
			}
			if($aprobacion['folio_final'] == $data->folio_final && $aprobacion['id_aprobacion'] != $data->id_aprobacion){
				$errors .= "- Folio final ya esta registrado para esa serie.\n";
				$ids .= ',#serie, #folio_final';
			} else {
				if($data->folio_final < $aprobacion['folio_final'] && $aprobacion['id_aprobacion'] != $data->id_aprobacion){
					$errors .= "- Folio final se encuentra en un rango ya registrado para esa serie.\n";
					$ids .= ',#serie, #folio_final';
				}
			}
		}
		
		if(empty($data->numero_aprobacion)){
			$errors .= "- Número de aprobación no puede estar vacio.\n";
			$ids .= ',#numero_aprobacion';
		} else {
			if(is_nan($data->numero_aprobacion)){
				$errors .= "- Número de aprobación debe ser un dato numerico.\n";
				$ids .= ',#numero_aprobacion';
			}
		}
		
		$ap = true;
		if(empty($data->fecha_aprobacion)){
			$ap = false;
			$errors .= "- Fecha de aprobación no puede estar vacio.\n";
			$ids .= ',#fecha_aprobacion';
		}
		
		if(empty($data->vigencia_aprobacion)){
			$ap = false;
			$errors .= "- Vigencia de aprobación no puede estar vacio.\n";
			$ids .= ',#vigencia_aprobacion';
		}
		
		if($ap){
			if($data->fecha_aprobacion > $data->vigencia_aprobacion){
				$errors .= "- Fecha de aprobación no puede ser mayor a la fecha de vigencia.\n";
				$ids .= ',#fecha_aprobacion,#vigencia_aprobacion';
			}
		}
		
		if(empty($data->serie)){
			$errors .= "- Serie no puede estar vacio.\n";
			$ids .= ',#serie';
		}
		
		if(empty($data->folio_inicial)){
			$errors .= "- Folio inicial no puede estar vacio.\n";
			$ids .= ',#folio_inicial';
		} else {
			if(is_nan($data->folio_inicial)){
				$errors .= "- Folio inicial debe ser un dato numerico.\n";
				$ids .= ',#folio_inicial';
			} else {
				$folio_inicial = $data->folio_inicial;
			}
		}
		
		if(empty($data->folio_final)){
			$errors .= "- Folio final no puede estar vacio.\n";
			$ids .= ',#folio_final';
		} else {
			if(is_nan($data->folio_final)){
				$errors .= "- Folio_final debe ser un dato numerico.\n";
				$ids .= ',#folio_final';
			} else{
				$folio_final = $data->folio_final;
			}
		}
		
		if(!empty($data->cantidad)){
			if(is_nan($data->cantidad)){
				$errors .= "- Cantidad debe ser un dato numerico.\n";
				$ids .= ',#cantidad';
			} else {
				if($folio_inicial < $folio_final){
					$intervalo = $folio_final - $folio_inicial + 1; 
					if($data->cantidad != $intervalo){
						$errors .= "- Intervalo de folio inicial y final no coincide con cantidad.\n";
						$ids .= ',#cantidad,#folio_inicial,#folio_final';
					}
				} else {
					$errors .= "- Folio inicial debe ser menor al folio final.\n";
					$ids .= ',#folio_inicial,#folio_final';
				}
			}
		}
		
		

		$result['ok'] = empty($errors);
		$result['error']['ids'] = preg_replace('/^,(.*)/','$1',$ids);
		$result['error']['msg'] = $errors;

		return array_utf8Decode($result);
	}

	/**
	 * Registra una nueva aprobacion
	 * @param $data Array que contiene la informacion a registrar
	 * @return array
	 */
	public function insAprobacion($data){
		$row = array_shift($this->executeSP('stp_insAprobacion(:id_usuario,:id_tipo_comprobante,:numero_aprobacion,:fecha_aprobacion,:vigencia_aprobacion,:serie,:folio_inicial,:cantidad,:folio_final)', $data));
		return $row;
	}

	/**
	 * Busca un registro con la coincidencia en la base de datos
	 * @param $data Serie a buscar
	 * @return array
	 */
	public function buscaAprobacion($data){
		$row = array_shift($this->executeSP('stp_validaAprobacion(:serie)', $data));
		return $row;
	}

	/**
	 * Actualiza la aprobacion especificada
	 * @param $data Array que contiene la informacion y Id del registro a actualizar
	 * @return array
	 */
	public function updAprobacion($data){
		$row = array_shift($this->executeSP('stp_updAprobacion(:id_aprobacion,:id_usuario,:id_tipo_comprobante,:numero_aprobacion,:fecha_aprobacion,:vigencia_aprobacion,:serie,:folio_inicial,:cantidad,:folio_final)', $data));
		return $row;
	}

	/**
	 * Elimina el registro especificado
	 * @param $data Valor entero, Id del registro
	 * @return array
	 */
	public function delAprobacion($data){
		$this->executeSP('stp_delAprobacion(:id)', array('id' => $data));
	}

	/**
	 * Registra el documento de aprobacion
	 * @param $data Array que contiene el id de la aprobacion y el documento aprobacion
	 * @return array
	 */
	public function addArchivoAprobacion($data){
		$row = array_shift($this->executeSP('stp_insArchivoAprobacion(:id_aprobacion,:documento_aprobacion)', $data));
		return $row;
	}

}
?>