<?php

/**
 * Clase DAO que usa los objetos creados de la clase que hereda
 * @author Eliud Trejo
 */
require FS_CLASSES . "Perfiles.php";

class PerfilesDAO extends Perfiles{
	/**
	* Funcion que ejecuta el store procedure devolviendo un array con la lista de todos elementos
	* @return array
	*/	
	public function perfilesPorTipo($tipo)
	{
		return $this->executeSP('stp_Perfiles(:tipo)', array('tipo' => $tipo));
	}
	
	/**
	* Funcion que ejecuta el store procedure devolviendo un array con la lista de todos elementos
	* @return array
	*/	
	public function listarPerfiles()
	{
		return $this->executeSP('stp_buscaPerfil(NULL)');
	}
	
 /**
  * Funcion que ejecuta la consulta de todos los datos 
  * y devuelve solo la que tenga el id especificado
  * @param var $id
  * @return array
  */
	public function buscarPerfil($id = null)
	{
		$row = array();
		$row = $this->executeSP('stp_buscaPerfil(:id_perfil)', array('id_perfil' => $id));
		$row = array_shift($row);

		return $row;
	}
	
 /**
  * Funcion que actualiza el estatus de un registro
  * @param array $data
  * @return array
  */
	public function statusPerfil( $data ){
		$sql = "stp_registraStatusPerfil(:id_perfil, :activo)";
		
		return $this->executeSP($sql, $data);
	}
	
 /**
  * Funcion que actualiza un registro ejecutando el store procedure correspondiente
  * @param array $data
  * @return array
  */
	public function updatePerfil($data)
	{
		$result = false;
		$sql = "stp_updPerfil(:id_perfil, :tipo_perfil, :descripcion, :permisos, :activo)";

		return $this->executeSP($sql, $data);
	}
/**
 * Funcion que ejecuta la insercion de datos,
 * por medio de la ejecucion de el store procedure correspondiente
 * @param var $data
 * @return array
 */
	public function insertPerfil($data)
	{
		$result = false;

		//-- Insertamos registro de perfiles
		$sql = "stp_updPerfil(:id_perfil, :tipo_perfil, :descripcion, :permisos, :activo)";

		return array_shift($this->executeSP($sql, $data));
	}

	/**
	 * Metodo Publico, Validacion de datos requeridos.
	 * @param array $data
	 * KEYS:
	 * <UL>
	 *	<LI> descripcion : string </LI>
	 * </UL>
	 * @return array
	 * KEYS:
	 * <UL>
	 *	<LI> ok : true | false </LI>
	 *	<LI>
	 *		error : array
	 *		<UL> KEYS
	 *			<LI> ids : string </LI>
	 *			<LI> msg : string </LI>
	 *		</UL>
	 *	</LI>
	 * </UL>
	 */
	public function validateData($data)
	{
		$data = (object) $data;
		$result = false;
		$errors = '';
		$ids = '';

		if ( empty( $data->descripcion ) ) {
			$errors .= '- Descripción no puede estar en blanco' . chr(13);
			$ids .= ',#descripcion';
		}

		if ( empty( $data->permisos ) ) {
			$errors .= '- Debe marcar por lo menos un permiso' . chr(13);
			$ids .= '';
		}

		$result['ok'] = empty($errors);
		$result['error']['ids'] = $ids;
		$result['error']['msg'] = $errors;

		return array_utf8Decode($result);
	}
 /**
  * Funcion que construye el arbol de permisos asignados a un perfil, para
  * devolverlos con checks que permitiran activarlos o desactivarlos
  * @param var $permisos
  * @return object
  */
	public function buildTree($permisos = '',$disabled=null)
	{
		$oXml = new DOMDocument();
		$checks = '';
		$this->setChecks( $checks );
		
		if ($oXml->load(FS_CONFIG . 'permisos.xml')) {
			$this->set_permisos($permisos);
			$this->setChecks($this->buildElements($oXml->documentElement,'',$disabled));
		}

		return $this->getChecks();
	}
 /**
  * Funcion que construye el arbol de permisos asignados a un perfil, para
  * devolverlos con checks que permitiran activarlos o desactivarlos
  * retorna una cadena para inyectarla en un elemento html
  * @param $oXml, $path = ''
  * @return string 
  */
	function buildElements($oXml, $path = '',$disabled=null)
	{
		$_build = '<ul>';

		foreach ($oXml->childNodes as $node) {
			if ($node->nodeType == XML_ELEMENT_NODE) {
				$_name = trim($node->getAttribute('id'));
				$_id = 'chk_' . $_name;
				$_text = utf8_decode($node->getAttribute('text'));

				$_checked = preg_match('/\[' .  $_name . '\|/', $this->get_permisos()) > 0 
																? ' checked="checked"' : '';

				$_build .= '<li>'
					. '<input type="checkbox" '. 
						$disabled . 
						' id="' . $_id . '" 
							name="form[permisos][]" 
							value="' . $_name . '" 
								class="' . $path . '"' . $_checked . '>'
					. '<label for="' . $_id . '" class="' . $path . '">' . $_text . '</label>'
					. '</li>';

				if ($node->hasChildNodes()) {
					$_build .= $this->buildElements($node, trim($path . ' ' . $_name),$disabled);
				}
			}
		}

		$_build .= '</ul>';

		return $_build;
	}
 /**
  * Funcion que construye el arbol de permisos asignados a un perfil, para
  * devolverlos con checks que permitiran activarlos o desactivarlos
  * retorna una cadena para inyectarla en un elemento html
  * @param $Xml
  * @return string 
  */
	public function buildElements2( $xml ){
		$this->addChecks('<ul class="permisos">');
		$xxx = '';
		for($i=0; $i < count($xml->child); $i++){

			$checked = '';

			if ($this->get_permisos()){
				//-- Verifica si permiso se encuentra "activado" en el perfil actual
				if (strpos($this->get_permisos(), '[' . trim($xml->child[$i]->permiso) . '|') !== false){
					$checked = ' checked="checked"';
					$xxx .= $xml->child[$i]->permiso . ",\n";
				}
			}

			$checks .= '<li>';

			if(isset( $xml->child[$i]->parent )){
				$checks .= 
					'<img class="tree" align="absmiddle" _action="true" 
						_ruta="'.WS_ROOT.'css/img/" src="'.WS_ROOT.'css/img/contract.gif"/>';
			}else{
				$checks .= '<img src="'.WS_ROOT.'css/img/blank.gif" style="width: 18px;"/>';
			}

			$checks .= '<input type="checkbox" id="' . 
							$xml->child[$i]->permiso . '" name="form[permisos][]" 
								value="' . $xml->child[$i]->permiso . '"' . 
								$checked . '>' //-- Checkbox
				. '<label class="' . 
						$classes . '" 
							for="' . $xml->child[$i]->permiso . '">' . 
					utf8_decode($xml->child[$i]->descripcion) . '</label>'; //-- Etiqueta

			if(isset( $xml->child[$i]->parent )){
				$this->buildElements($xml->child[$i]->parent);
			}
			$checks .= "</li>";
		}

		$checks .= '</ul>';
		$this->addChecks($checks);
		echo $xxx;
	}
 /**
  * Funcion que verifica y retorna los permisos que tiene un usuario en base a su perfil
  * @param $id_user
  * @return object 
  */
	public function permisosUsuario($id_user = null)
	{
		if (is_null($id_user)) $id_user = $this->id_usuario;

		$sql = "stp_validaPerfilUsuarios(:id_user)";
		$perfiles = $this->executeSP($sql, array('id_user' => $id_user));
		$permisos = $perfiles[0]['permisos'];
		$this->set_permisos($permisos);
//		debug($permisos);
		return $this->get_permisos();
	}
 /**
  * Funcion que verifica si se tiene permiso para accesar a algun elemento de la aplicacion
  * @param $key
  * @return 
  */
	public function tienePermiso($key)
	{
		return (preg_match('/\[' . $key . '\|/', $this->get_permisos()) > 0 || false);
	}
	
 /**
  * Funcion que transforma un elemento DHTMLX a un elemento xml
  * @param $menuXML
  * @return $xml
  */
	public function parseDHTMLXMenu($menuXML)
	{
		$xml = new DOMDocument();
		if (! $xml->loadXML($menuXML)) $die = die('Problema con el XML del Menu');

		$this->traverseMenu($xml->documentElement, $xml->documentElement);
		return $xml->saveXML();
	}

	/**
	 * Funcion que adhiere un elemento al menu
	 * @param type $root
	 * @param type $element
	 */
	private function traverseMenu($root, &$element)
	{
		$menuItems = $element->childNodes;
		$n = $menuItems->length;

		for ($i = $n; $i >= 0; $i--) {
			$node = $menuItems->item($i);

			if ($node->nodeType == XML_ELEMENT_NODE) {
				$id = trim($node->getAttribute('id'));
				$attr = $node->getAttribute('text');
				$persistent = $node->getAttribute('persistent');

				if ($persistent || $this->tienePermiso($id)) {
					if ($node->hasChildNodes()) $this->traverseMenu($root, $node);
				} else {
					if ($node->nodeName == 'item') $element->removeChild($node);
				}
			}
		}
	}
}
?>