/*===================================================================================================*/
/*   Helper de JavaScript para apoyar la validacion de los inputs y de las formas de la aplicacion.  */
/*===================================================================================================*/

//
var errorTemp = '';
var listaErr = '';
var errorLong = new Array();


//es mas sencillo y rapido checar sin RegEx para esta comprobacion.
function isEmpty(cadena){
	$trimmed = $.trim(cadena);
	return ($trimmed=="")?true:false;
}

function validatorTrimmer(cadena)
{
	$trim = cadena.replace(/^\s+/g,'').replace(/\s+$/g,'');
	return cadena;
}

/* Este arreglo tiene todas las clases que se checaran, este es el orden:
 *		nombre de la clase
 *		mensaje de error pequeño que se situa al lado dle campo input
 *		mensaje de error grande y mas descrpitivo que se situara en un acumulativo.
 *		la expresion regular que se comprobara, si la cadena evaulada hace match con esta expresion regular regresara true.
 *	Si necesitas declarar un nuevo criterio(clase) para evaluar solo agregalo 
 *	de la mimsa manera que los anteriores y hara la comprobacion correspondiente.
 */
var strError= [
	['reqStatus',
		'campo Obligatorio',
		'Campo no puede estar vacío, es requerido para aprobación de la solicitud.',
		/^.*$/g
	],
	['required',
		'campo Requerido',
		'Campo no puede estar vacío.',
		/^.*$/g
	],
	['nonZero',
		'Valor incorrecto',
		'El valor no puede ser igual a 0.',
		/^(\d+)$/g
	],
	['date-mx',
		'fecha incorrecta',
		'Debe indicar una fecha válida en formato dd/mm/aaaa',
		/^(\d{2}\/\d{2}\/\d{4})?$/gi
//		/^(\d{2})\/(0[1-9]|1[012])\/(0[1-9]|[12][0-9]|3[01])$/
	],
	['mes',
		'Mes incorrecto',
		'Debes indicar un mes válido',
		/^(0[1-9]|1[01])?$/
	],
	['numeric',
		'solo números',
		'Sólo se aceptan números enteros.',
		/^(\d+)?$/g
	],
	['alphanumeric',
		'solo alfanumericos',
		'Sólo se aceptan letras y números.',
		/^([a-zA-ZáéíóúüñÜÑ\s\d\.,]+)?$/gi
//		/^([\w\.,\s]+)?$/gi
		/* anterior RegEx
		 *  /^((\w+(\s?\w+)*)+)?$/gi 
		 */
	],
	['float',
		'solo flotantes',
		'Solo se aceptan números con/sin decimales.',
		/^((\d+)?((\.)(\d+))?)?$/g
	],
	['money',
		'solo moneda',
		'Formato del campo debe ser de tipo Moneda.',
		/^(-?\$(((\d{1,3},)+\d{3})|\d+)\.\d*)?$/
	],
	['email',
		'solo e-mail',
		'Debe contener una dirección de correo electrónico estandar.',
		/^([\w\._]+@[\w-w\.]+\.[\w]{2,3})?$/gi
		/* Anterior y muy complicado RegEx 
		 *^([a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+(?:[A-Z]{2}|com|org|net|edu|gov|mil|biz|info|mobi|name|aero|asia|jobs|museum))?$/g
		 */
	],
	['RFC',
		'solo RFC',
		'Formato no corresponde a RFC para Personas Física ó Moral.',
		/^(([A-Za-z]{3,4})(\d{2})(0[1-9]|1[012])(0[1-9]|[12][0-9]|3[01])(\w{3}))?$/gi
	],
	['RFCFisica',
		'solo RFC fisica',
		'Formato incorrecto para RFC de Persona Física.',
		/^(([A-Za-z]{4})(\d{2})(0[1-9]|1[012])(0[1-9]|[12][0-9]|3[01])(\w{3}))?$/gi
	],
	['RFCMoral',
		'solo RFC moral',
		'Formato incorrecto para RFC de Persona Moral.',
		/^(([A-Za-z]{3})(\d{2})(0[1-9]|1[012])(0[1-9]|[12][0-9]|3[01])(\w{3}))?$/gi
	],
	['phone',
		'Formato teléfono incorrecto',
		'Sólo se aceptan 7, 8 o 10 dígitos para télefonos fijos.',
//		/^(\d{7,8,10})?$/
		/^(\d{7}|\d{8}|\d{10}|)?$/
//		/^(((\d{2} )?\d{4}-\d{4})|((\d{3} )?\d{3}-\d{4}))?$/gi
	],
	['cell-phone',
		'formato teléfono incorrecto',
		'Télefono celular debe ser de 10 dígitos.',
		/^(\d{10})?$/
//		/^(((\d{2} )?\d{4}-\d{4})|((\d{3} )?\d{3}-\d{4}))?$/gi
	],
	['alphabetic',
		'sólo letras y espacios',
		'Unicamente son permitidos letras y espacios.',
		/^([a-zA-ZáéíóúüñÜÑ\s\d\.,]+)?$/gi
	],
	['CURP',
		'formato incorrecto',
		'CURP no tiene el formato correcto.',
		/^([A-Za-z]{4}\d{6}[HhMm][A-Za-z]{5}[A-Za-z0-9]{2})?$/
	],
	['licencia',
		'formato incorrecto',
		'No. de licencia debe ser de 7 dígitos.',
		/^(\d{7})?$/
	],
	['percent',
		'formato porcentaje',
		'Formato del campo debe ser porcentaje.',
		/^((100|\d{0,2})(\.\d{2})?\%?)?$/
	],
	['percentNDecimal',
		'formato porcentaje con n ',
		'Formato del campo debe ser porcentaje.',
		/^((100|\d{0,2})(\.\d*)?\%?)?$/
	],
	['user-id',
		'ID de usuario',
		'ID de usuario debe empezar con una letra y debe tener 6 a 16 caracteres (letras y numeros).',
		/^[a-zA-Z][a-zA-Z0-9]{4,16}$/
	],
	['CP',
		'Código Postal',
		'El código postal debe ser de 5 dígitos. Las posiciones a la izq. deben rellenarse con cero.',
		/^\d{5}$/
	]
];

/*esta funcion espera a que la pagina se haya cargado en su totalidad.*/
$(document).ready(
	function () {
		//TODO: Tomar valores desde el arreglo principal de errores.
		$('.restrict').live('keydown',function(e){
			var input = $(this);
			var key = e.which;

			if(input.hasClass('numeric')){
				if ((key!=8 && key!=9 && key!=0 && key!=13) && (key < 46 || key > 57))
					e.preventDefault();
			}
			if(input.hasClass('money')||input.hasClass('float')||input.hasClass('percent')){
				if ((key!=8 && key!=9 && key!=0 && key!=13 && key!=190) && (key < 46 || key > 57))
					e.preventDefault();
			}
		});
		
		//-- Inicializa eventos para campos con formato de moneda
		$('.money').live(
			{
				'blur' : function () {
					formatMoney(this, false);
				},
				'focus' : function () {
					if ($(this).attr('readonly') != 'readonly') {
						unformatMoney(this);
					}
				}
			}
		);
			
		$('.percent').live(
			{
				'blur' : function () {
					formatPercent(this, false);
				},
				'focus' : function () {
					if ($(this).attr('readonly') != 'readonly') {
						unformatPercent(this);
					}
				}
			}
		);

		$('.percentNDecimal').live(
			{
				'blur' : function () {
					formatPercentNDecimal
					(this, false);
				},
				'focus' : function () {
					if ($(this).attr('readonly') != 'readonly') {
						unformatPercent(this);
					}
				}
			}
		);

		//-- Inicializa eventos para campos con formato de moneda
		$('.numeric').live(
			{
				'blur' : function () {
					formatNumeric(this);
				},
				'focus' : function () {
					unformatNumeric(this);
				}
			}
		);

		//-- Inicializa eventos para campos con formato de Código Postal
		$('.CP').live(
			{
				'blur' : function () {
					formatCP(this);
				}
			}
		);

		//-- Inicializa eventos para campos con formato de moneda
		$('.float').live(
			{
				'blur' : function () {
					formatFloat(this);
				},
				'focus' : function () {
					unformatFloat(this);
				}
			}
		);

		//-- Handler para campos solo mayúsculas
		$('.toUpper').live(
			{
				'blur' : function () {
					$(this).val($(this).val().toUpperCase());
				}
			}
		);

		//-- Handler para campos solo minúsculas
		$('.toLower').live(
			{
				'blur' : function () {
					$(this).val($(this).val().toLowerCase());
				}
			}
		);
		
		//-- Inicializa eventos para validacion de formatos en general
		$('input[type="text"]:enabled').live('change blur',function(){
			var input = $(this);
			input.val($.trim(input.val()));
			(checkForErrors(input))?cleanError(input):throwError(input);
		});	

		//-- Limpiar error al seleccionar radios
		$(':radio').live('change',
			function () {
				var group = $(':radio[name="' + this.name + '"]');
				group.filter(':first').removeClass('errorField');

				//-- Marcado/desmarcado de labels de radios con error
				group.each(
					function () {
						$('label[for="' + this.id + '"]').removeClass('radioError');
					}
				);
			}
		);

		//-- Limpiar error Checkbox
		$('.checkRequired input:checkbox').live('change',function(){
			if ($(this).parent().find('input:checkbox').is(':checked')) {
				$(this).parent().find('label').removeClass('radioError');
			} else {
				$(this).parent().find('label').addClass('radioError');
			}
		});
		
		setFormats();
	}
);

/* Esta funcion recibe un objeto Input y checa si este objeto tiene alguna de las clases del validator, 
 * si posee alguna checa que el contenido este de acuerdo con el regex que le corresponde.
 * Respondera con un False si ha encontrado un error en alguna comprobacion.
 * y regresara un True cuando no haya ningun error en el input entregado.
 */
function checkForErrors(input, checkRequired){
	var valid = true;

	//-- Requeridos por cambio de Estatus de la solicitud a aprobación.
	if (checkRequired != null && checkRequired) {
		if (input.hasClass('reqStatus')) {
			if (isEmpty(input.val())) {
				valid = false;
				errorTemp = strError[0][1];
				errorLong.push(strError[0][2]);
			} else if (input.hasClass('nonZero')) {
			var value = parseFloat(unformatMoney(input.val()));

			if(value == 0) {
				valid = false;
				errorTemp = strError[2][1];
				errorLong.push(strError[2][2]);
			}
		}
		}
	}

	//-- Requeridos en el guardado individual de un Tab.
	if(input.hasClass('required')){
		if(isEmpty(input.val())){
			valid = false;
			errorTemp = strError[1][1];
			errorLong.push(strError[1][2]);
		} else if (input.hasClass('nonZero')) {
			var value = parseFloat(unformatMoney(input.val()));

			if(value == 0) {
				valid = false;
				errorTemp = strError[2][1];
				errorLong.push(strError[2][2]);
			}
		}
	}

	//empieza en 3 porque los primeros elementos del objeto son comprobaciones idependientes.
	for (i = 3; i < strError.length; i++) {
		if (input.hasClass(strError[i][0])) {
			if (! input.val().match(strError[i][3])) {
				valid = false;
				errorTemp = strError[i][1];
				errorLong.push(strError[i][2]);
			}
		}
	}
	
	//-- Revisa que fecha sea válida
	if (input.hasClass('date-mx')&& input.val().trim() != '' && ! fechaValida(input.val())) {
		valid = false;
		errorTemp = strError[3][1];
		errorLong.push(strError[3][2]);
	}

	return valid;
}

function cleanError(input){
	obj = input;
	obj.removeClass('errorField');
//	eIcon = obj.parent().find('.errorIcon');
//	sError = obj.parent().find('.strError');
//	eIcon.remove();
//	sError.remove();
	input.removeAttr('title');
}

function throwError(input){
	obj = input;
	obj.addClass('errorField');
//	parrafo = obj.parent();
//	eIcon = parrafo.find('.errorIcon');
//	eIcon.remove();
//	sError = parrafo.find('.strError');
//	sError.remove();
//	parrafo.append('<div class="errorIcon"></div>');
//	parrafo.append('<div class="strError">'+errorTemp+'</div>');
	input.attr('title', errorLong[errorLong.length - 1]);
	errorTemp='';
}

function validate(tab, quiet){
	var result = true;
	listaErr = '';
	
	$(tab + ' input[type="text"]:not([name*="[0]"]):enabled,'
			+ tab + ' select:not([name*="[0]"]):enabled, '
			+ tab + ' textarea:not([name*="[0]"]):enabled').each(
		function () {
			var input = $(this);

			if (checkForErrors(input)) {
				cleanError(input);
			} else {
				throwError(input);
				result = false;
			}
		}
	);

	//-- Validacion Checkbox
	if($(tab + " .checkRequired input:checkbox").length){
		if ($(tab + " .checkRequired").find('input:checkbox').is(':checked')) {
			$(tab + " .checkRequired").find('label').removeClass('radioError');
		} else {
			result = false;
			$(tab + " .checkRequired").find('label').addClass('radioError');
		}
	}

	//-- validacion de input[type=radio]
	var radios = new Array();
	$(tab + ' .checkRequired :radio:not([name*="[0]"]):enabled').each(
		function() {
			if (radios.search(this.name) == -1) {
				var item_name = '[name="' + this.name + '"]';
				radios.push(item_name);
				var group = $(this).closest('form').contents().find(':radio').filter(item_name);
				var checked = group.filter(':checked').length;

				if (checked == 0) {
					group.filter(':first').addClass('errorField');
					result = false;
				} else {
					group.filter(':first').removeClass('errorField');
				}

				//-- Marcado/desmarcado de labels de radios con error
				group.each(
					function () {
						if (checked == 0) {
							$('label[for="' + this.id + '"]').addClass('radioError');
						} else {
							$('label[for="' + this.id + '"]').removeClass('radioError');
						}
					}
				);
			}
		}
	);
	
	if (! result && (quiet == null || quiet == false)) {
		/* aun en desarrollo
		for(i=0;i<errorLong.length;i++){
			$('.errorValidator').append('<p>'+errorLong[i]+'</p>');
		}
		$('.errorValidator').show();
		setTimeout(function(){ $('.errorValidator').hide(); }, 3000);
		*/
		var numErrs = $(tab + ' .errorField').length;
		alert('No es posible guardar la información capturada.\n\n'
			+ 'Se encontraron ' + numErrs + ' campo(s) con errores.');
	}
	
	return result;
}

function validateAll(quiet){
	var result = true;
	listaErr = '';

	$('form input[type="text"]:not([name*="[0]"]):enabled,'
			+ 'form select:not([name*="[0]"]):enabled,'
			+ 'form textarea:not([name*="[0]"]):enabled').each(
		function () {
			var input = $(this);
			
			if (checkForErrors(input, true)) {
				cleanError(input);
			} else {
				nombreforma = input.closest('form').attr('name');
				idInput = input.attr('id');
				listaErr +='el input: '+idInput+ ' que esta en: '+ nombreforma+' tiene error. \n';
				throwError(input);
				result = false;
			}
		}
	);

	//-- validacion de input[type=radio]
	var radios = new Array();
	$(':radio:not([name*="[0]"]):enabled').each(
		function() {
			if (radios.search(this.name) == -1) {
				var item_name = '[name="' + this.name + '"]';
				radios.push(item_name);
				var group = $(this).closest('form').contents().find(':radio').filter(item_name);
				var checked = group.filter(':checked').length;

				if (checked == 0) {
					group.filter(':first').addClass('errorField');
					result = false;
				} else {
					group.filter(':first').removeClass('errorField');
				}

				//-- Marcado/desmarcado de labels de radios con error
				group.each(
					function () {
						if (checked == 0) {
							$('label[for="' + this.id + '"]').addClass('radioError');
						} else {
							$('label[for="' + this.id + '"]').removeClass('radioError');
						}
					}
				);
			}
		}
	);

	if (! result && (quiet == null || quiet == false)) {
	   var numErrs = $('.errorField').length;
	   
		alert('No es posible guardar la información capturada.\n\n'
			+ 'Se encontraron ' + numErrs + ' campo(s) con errores.');
//		alert(listaErr);
	}
	
	return result;
}

function formatMoney(oSrc, blank)
{
	var retVal = false;
	
	if (typeof oSrc != 'object') {
		src = oSrc.toString();
		retVal = true;
	} else {
		src = oSrc.value.trim();
	}

	if (src != '') {
		negativo = /^-/.test(src) ? '-' : '';
		cadena = src.replace(/[^\d\.]/g, '');
		paso = cadena.split(/\./);

		if (paso.length == 1) decimales = "000000";
		else decimales = paso[1].pad(6, "0", 1); //.substring(0, 2);

		entero = paso[0].reverse().replace(/(\d{3})/g, '$1,').reverse().replace(/^,(.*)/, '$1');
		entero = (entero == '' ? '0' : entero);
		
		_money = negativo + "$" + entero + "." + decimales;

		if (blank) _money = _money.replace(/^\$0\.00$/, '');
		
		if (retVal) {
			return _money;
		} else {
			oSrc.value = _money;
		}
		
	}
}

function unformatMoney(oSrc)
{
	if (typeof oSrc == 'string') {
		src = oSrc.replace(/,|\$/g, '');
		return src;
	} else {
		src = oSrc.value.replace(/,|\$/g, '');
		oSrc.value = src;
		oSrc.select();
	}
}

function formatPercent(oSrc, blank)
{
	src = oSrc.value.trim();

	if (src != '') {
		negativo = /^-/.test(src) ? '-' : '';
		cadena = src.replace(/[^\d|\.]/g, '');
		paso = cadena.split(/\./);

		if (paso.length == 1) decimales = "00";
		else decimales = paso[1].pad(2, "0", 1).substring(0, 2);

		entero = paso[0].reverse().replace(/(\d{3})/g, "$1,").reverse().replace(/^,(\d{3})/,"$1");
		entero = entero == '' ? '0' : entero;

		oSrc.value = negativo + entero + "." + decimales+ "%";

		if (blank) oSrc.value = oSrc.value.replace(/^0\.00\%$/, '');
	}
}

function formatPercentNDecimal(oSrc, blank)
{
	src = oSrc.value.trim();

	if (src != '') {
		negativo = /^-/.test(src) ? '-' : '';
		cadena = src.replace(/[^\d|\.]/g, '');
		paso = cadena.split(/\./);
		
		if (paso.length == 1) decimales = "000000";
		else decimales = paso[1].pad(6, "0", 1);

		entero = paso[0].reverse().replace(/(\d{3})/g, "$1,").reverse().replace(/^,(\d{3})/,"$1");
		entero = entero == '' ? '0' : entero;

		oSrc.value = negativo + entero + "." + decimales+ "%";

		if (blank) oSrc.value = oSrc.value.replace(/^0\.00\%$/, '');
	}
}
function unformatPercent(oSrc)
{
	if (typeof oSrc == 'string') {
		src = oSrc.replace(/,|\%/g, '');
		return src;
	} else {
		src = oSrc.value.replace(/,|\%/g, '');
		oSrc.value = parseFloat(src) == 0 ? '' : src;
		oSrc.select();
	}
}

function formatNumeric(oSrc)
{
	src = oSrc.value.trim();
    src = src.replace(/[^\d]/g, '');
//	src = isEmpty(src) ? '0' : src;
    oSrc.value = src;
}

function unformatNumeric(oSrc)
{
	if (typeof oSrc == 'string') {
		src = oSrc;
		return src;
	} else {
		src = oSrc.value;
		oSrc.value = src;
		oSrc.select();
	}
}

function formatCP(oSrc)
{
	formatNumeric(oSrc);
	src = oSrc.value.trim().pad(5, '0');
	oSrc.value = src;
}

function formatFloat(oSrc)
{
	src = oSrc.value.trim();

	if (src != '' && ! $(oSrc).hasClass('errorField')) {
		var nums = src.replace(/[^\.\d-]/g, '').split(/\./);
		
		//-- Decimales
		var decimals = 2;
		var decClass = oSrc.className.match(/precision-\d+/);
		
		if (decClass != null) {
			decimals = Math.max(1, parseInt(decClass[0].replace(/precision-(\d+)/, '$1')));
		}
		
		nums[1] = (nums[1] == null || nums[1] == '' ? '0' : nums[1]);
		nums[1] = nums[1].substr(0, decimals).pad(decimals, '0', 1)
		
		//-- Enteros
		var length = $(oSrc).attr('maxlength');
		nums[0] = (nums[0] == null || nums[0] == '' ? '0' : nums[0]);
		
		src = nums[0] + '.' + nums[1];
		if (length != null) src = src.slice(-length);
	}

    oSrc.value = src;
}

function unformatFloat(oSrc)
{
	if (typeof oSrc == 'string') {
		src = oSrc;
		return src;
	} else {
		src = oSrc.value;
		oSrc.value = src;
		oSrc.select();
	}
}

/**
 * Inicializa todas las clases con sus 
 * diferentes validaciones y formatos especificados
 */
function setFormats()
{
	//-- Inicializa campos con formato de moneda
	$('.money').each(
		function () {
			formatMoney(this, false);
		}
	);

	//-- Inicializa campos con formato de procentaje
	$('.percent').each(
		function () {
			formatPercent(this, false);
		}
	);
	
	//-- Inicializa campos con formato de procentaje
	$('.percentNDecimal').each(
		function () {
			formatPercentNDecimal(this, false);
		}
	);

//	//-- Inicializa campos con formato de numerico
//	$('.numeric').each(
//		function () {
//			formatNumeric(this);
//		}
//	);

	//-- Inicializa campos con formato de flotante
	$('.float').each(
		function () {
			formatFloat(this);
		}
	);
	
	//-- Inicializa campos con formato de Código Postal
	$('.CP').each(
		function () {
			formatCP(this);
		}
	);
	
	//-- Inicializa campos con formato de fecha
	$('.date-mx:enabled:not([readonly])').each(
		function () {
			$(this).datepicker();
		}
	);
	
	$('select.readonly:enabled').each(
		function () {
			$(this).mousedown(
				function (e) {
					e.preventDefault();
				}
			)
		}
	);
}
